package zero1hd.rhythmbullet.audio;

import java.io.IOException;

import org.jaudiotagger.audio.exceptions.InvalidAudioFrameException;
import org.jaudiotagger.audio.exceptions.ReadOnlyFileException;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.tag.TagException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.GdxRuntimeException;

import javazoom.jl.decoder.Bitstream;
import javazoom.jl.decoder.BitstreamException;
import javazoom.jl.decoder.DecoderException;
import javazoom.jl.decoder.Header;
import javazoom.jl.decoder.MP3Decoder;
import javazoom.jl.decoder.OutputBuffer;

public class Mp3AudioData implements AudioData {
	private int readWindowSize = 1024;

	private Music playbackMusic;
	private int readIndex;
	
	private int sampleRate;
	private long sampleCount;
	private float durationInSeconds;
	
	Bitstream bitstream;
	MP3Decoder decoder;
	OutputBuffer sampleBuffer;
	Mp3.Music test;
	public Mp3AudioData(FileHandle audioFile) {
		try {
			MP3File mp3File = new MP3File(audioFile.file());
			sampleCount = mp3File.getMP3AudioHeader().getNumberOfFrames();
			durationInSeconds = mp3File.getMP3AudioHeader().getNumberOfFrames()/readWindowSize;
			sampleRate = mp3File.getMP3AudioHeader().getSampleRateAsNumber();
		} catch (IOException | TagException | ReadOnlyFileException | InvalidAudioFrameException e) {
			e.printStackTrace();
		}
		
		bitstream = new Bitstream(audioFile.read());
		decoder = new MP3Decoder();
		
		try {
			Header header = bitstream.readFrame();
			if (header == null) throw new GdxRuntimeException("Empty MP3");
			int channels = header.mode() == Header.SINGLE_CHANNEL ? 1 : 2;
			sampleBuffer = new OutputBuffer(channels, false);
			decoder.setOutputBuffer(sampleBuffer);
		} catch (BitstreamException e) {
			throw new GdxRuntimeException("error while preloading mp3", e);
		}
		
		playbackMusic = Gdx.audio.newMusic(audioFile);
		
		reset();
	}

	@Override
	public void readIndexUpdate() {
		readIndex = (int) (playbackMusic.getPosition() * sampleRate / readWindowSize);
	}

	@Override
	public int getReadIndex() {
		return readIndex;
	}

	@Override
	public void reset() {
		playbackMusic.stop();
	}

	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}

	@Override
	public Music getPlaybackMusic() {
		return playbackMusic;
	}

	@Override
	public long getFrameCount() {
		return sampleCount;
	}

	@Override
	public void dispose() {
		reset();
		playbackMusic.dispose();
		try {
			bitstream.close();
		} catch (BitstreamException e) {
			e.printStackTrace();
		}
	}

	@Override
	public float getDuration() {
		return durationInSeconds;
	}

	@Override
	public int readSamples(float[] samples) {
		int totalRead = 0;
		
		for (int sid = 0; sid < samples.length; sid++) {
			try {
				Header header = bitstream.readFrame();
				
				if (header == null) break;
				
				try {
					decoder.decodeFrame(header, bitstream);
				} catch (ArrayIndexOutOfBoundsException | DecoderException e) {
					System.out.println(e);
				}
				bitstream.closeFrame();
				sampleBuffer.reset();
				byte[] buffer = sampleBuffer.getBuffer();
				
				samples[sid] += (buffer[1] << 8) + (buffer[0] & 0x00ff);
				if (sampleBuffer.isStereo()) {
					samples[sid] += (buffer[3] << 8) + (buffer[2] & 0x00ff);
					samples[sid] /= 2;
				}
				totalRead++;
				
				samples[sid] /= Short.MAX_VALUE + 1;
			} catch (BitstreamException e1) {
				e1.printStackTrace();
			}
		}
		
		return totalRead;
	}

	@Override
	public float getSampleRate() {
		return sampleRate;
	}
	
	

}
